<?php

class ProjectdataController extends Zend_Controller_Action {
	
	var $project_service;
	var $role_service;
	var $user_service;
	
	public function init() {
		require_once (dirname ( __FILE__ ) . '/services/UserManagement.php');
		require_once (dirname ( __FILE__ ) . '/services/RoleManagement.php');
		require_once (dirname ( __FILE__ ) . '/services/ProjectManagement.php');
		
		$options = $this->getInvokeArg('bootstrap')->getOptions();
		$serverpath = $options['galaxy']['serverpath'];
		
		$this->project_service= new ProjectManagement ( $serverpath );
		$this->role_service = new RoleManagement ( $serverpath );
		$this->user_service = new UserManagement ( $serverpath );
	}
	
	public function indexAction() {
		$id = $this->_getParam ( 'id', 0 );
		$name = $this->_getParam ( 'name', "" );
		$description = $this->_getParam ( 'description', "" );
		$startDate = $this->_getParam ( 'startDate', "" );
		$endDate = $this->_getParam ( 'endDate', "" );
		$this->view->project = array (
				'id' => $id,
				'name' => $name,
				'description' => $description,
				'startDate' => $startDate,
				'endDate' => $endDate 
		);
		
		$this->view->roles = $this->role_service->findRolesByProject ( $id );
		
		
		
		$userroles = array();
		$users = $this->user_service->findParticipantsByProject ( $id );
		foreach($users as $user){ 
			$userroles[$user->id]= $this->role_service->findRolesByProjectAndParticipant($id,$user->id);
		}
		
		
		$this->view->users = $users;
		$this->view->userroles = $userroles;
	}
	
	public function addroleAction() {
		$form = new Application_Form_Selection ();
		$form->createContent ( true );
		$form->submit->setLabel ( 'Add Role' );
		
		$projectId = $this->_getParam ( 'id', 0 );
		
		$existingroles = array();	
		foreach ($this->role_service->findRolesByProject($projectId) as $role ) {
			$existingroles[$role->id] = $role;
		}
		
		$roledata = array ();
		foreach ( $this->role_service->findAllRoles () as $role ) {
			if(!isset($existingroles[$role->id])){
				$roledata [$role->id] = $role->name;
			}
		}
		
		$form->selection->setMultiOptions ( $roledata );
		$form->populate ( array (
				'id' => $projectId
		) );
		
		$this->view->form = $form;
		
		if ($this->getRequest ()->isPost ()) {
			$formData = $this->getRequest ()->getPost ();
			if ($form->isValid ( $formData )) {
				$roleid = $form->getValue ( 'selection' );
	
				$this->project_service->addRole ( $projectId, $roleid );
				
				$name = $this->_getParam ( 'name', "" );
				$description = $this->_getParam ( 'description', "" );
				$startDate = $this->_getParam ( 'startDate', "" );
				$endDate = $this->_getParam ( 'endDate', "" );
				$this->_helper->redirector ( 'index', 'projectdata', null, array (
						'id' => $projectId,
						'name' => $name,
						'description' => $description,
						'startDate' => $startDate,
						'endDate' => $endDate 
				) );
			} else {
				$this->view->error = ' * Champs requis.';
			}
		}
	
	}
	
	public function removeroleAction() {
		if ($this->getRequest ()->isPost ()) {
			$del = $this->getRequest ()->getPost ( 'del' );
			
			$projectId = $this->_getParam ( 'id', 0 );
			$name = $this->_getParam ( 'name', "" );
			$description = $this->_getParam ( 'description', "" );
			$startDate = $this->_getParam ( 'startDate', "" );
			$endDate = $this->_getParam ( 'endDate', "" );
			
			if ($del == 'Yes') {
				$roleId = $this->getRequest ()->getPost ( 'id' );
				$this->project_service->removeRole ( $projectId, $roleId );
			}
			
			$this->_helper->redirector ( 'index', 'projectdata', null, array (
					'id' => $projectId,
					'name' => $name,
					'description' => $description,
					'startDate' => $startDate,
					'endDate' => $endDate 
			) );
		} else {
			$id = $this->_getParam ( 'roleid', 0 );
			$name = $this->_getParam ( 'name', "" );
			$this->view->account = array (
					"id" => $id,
					"name" => $name 
			);
		}
	}
	
	
	
	
	public function adduserAction() {
		
		$projectId = $this->_getParam ( 'id', 0 );
		
		$form = new Application_Form_AddParticipant();
		$form->createContent ( true );
		$form->submit->setLabel ( 'Add Participant' );
		$form->selection->setLabel ( 'User' );
		$form->multiselection->setLabel ( 'Roles' );

		
		$userdata = array ();
		
		$existingusers = array();
		
		foreach ($this->user_service->findParticipantsByProject($projectId) as $user) {
			$existingusers[$user->id] = $user;
		}
		
		foreach ( $this->user_service->findAllParticipants () as $user) {
			if(!isset($existingusers[$user->id])){
				$userdata [$user->id] = $user->firstName . ' ' . $user->lastName;
			}
		}
		
		$form->selection->setMultiOptions ( $userdata );
		
		foreach ( $this->role_service->findRolesByProject($projectId) as $role ) {
			$form->multiselection->addMultiOption($role->id, $role->name);
		}
	

		$form->populate ( array (
				'id' => $projectId
		) );
	
		$this->view->form = $form;
	
		if ($this->getRequest ()->isPost ()) {
			$formData = $this->getRequest ()->getPost ();
			if ($form->isValid ( $formData ) && $formData['multiselection'] != null) {
				$userId = $form->getValue ( 'selection' );
				$roleIds = $form->getValue ( 'multiselection' );
				
				
				$this->project_service->addParticipant( $projectId, $userId,$roleIds);
	
				$name = $this->_getParam ( 'name', "" );
				$description = $this->_getParam ( 'description', "" );
				$startDate = $this->_getParam ( 'startDate', "" );
				$endDate = $this->_getParam ( 'endDate', "" );
				
				$this->_helper->redirector ( 'index', 'projectdata', null, array (
						'id' => $projectId,
						'name' => $name,
						'description' => $description,
						'startDate' => $startDate,
						'endDate' => $endDate
				) );
			} else {
				$this->view->error = ' * Champs requis.';
			}
		}
	
	}
	
	
	
	public function editeuserAction() {
	
		$projectId = $this->_getParam ( 'id', 0 );
		$userId = $this->_getParam ( 'userid', 0 );
	
		$form = new Application_Form_EditParticipant();
		$form->createContent ( true );
		$form->submit->setLabel ( 'Edit Roles of Participant' );
		$form->multiselection->setLabel ( 'Roles' );

		$allroles = $this->role_service->findRolesByProject($projectId);
		foreach ($allroles as $role ) {
			$form->multiselection->addMultiOption($role->id, $role->name);
		}
		
		
		$selectedrole = array();
		foreach ( $this->role_service->findRolesByProjectAndParticipant($projectId, $userId) as $role ) {
			$selectedrole[] = $role->id;
		}
		
	
	
		$form->populate ( array (
				'id' => $projectId,
				'multiselection' => $selectedrole
		) );
	
		$this->view->form = $form;
	
		if ($this->getRequest ()->isPost ()) {
			$formData = $this->getRequest ()->getPost ();
			if ($form->isValid ( $formData ) && $formData['multiselection'] != null) {
				$roleIds = $form->getValue ( 'multiselection' );
	
				$this->project_service->removeParticipant($projectId, $userId, $selectedrole);
				$this->project_service->addParticipant( $projectId, $userId,$roleIds);
	
				$name = $this->_getParam ( 'name', "" );
				$description = $this->_getParam ( 'description', "" );
				$startDate = $this->_getParam ( 'startDate', "" );
				$endDate = $this->_getParam ( 'endDate', "" );
	
				$this->_helper->redirector ( 'index', 'projectdata', null, array (
						'id' => $projectId,
						'name' => $name,
						'description' => $description,
						'startDate' => $startDate,
						'endDate' => $endDate
				) );
			} else {
				$this->view->error = ' * Champs requis.';
			}
		}
	
	}
	
	public function removeuserAction() {
		if ($this->getRequest ()->isPost ()) {
			$del = $this->getRequest ()->getPost ( 'del' );
				
			$projectId = $this->_getParam ( 'id', 0 );
			$name = $this->_getParam ( 'name', "" );
			$description = $this->_getParam ( 'description', "" );
			$startDate = $this->_getParam ( 'startDate', "" );
			$endDate = $this->_getParam ( 'endDate', "" );
				
			if ($del == 'Yes') {
				$userId = $this->getRequest ()->getPost ( 'id' );
				
				$roles= $this->role_service->findRolesByProjectAndParticipant($projectId,$userId);
				
				$roleIds = array();
				foreach($roles as $role){
					$roleIds[] = $role->id;
				}

				$this->project_service->removeParticipant ( $projectId, $userId,$roleIds );
			}
				
			$this->_helper->redirector ( 'index', 'projectdata', null, array (
					'id' => $projectId,
					'name' => $name,
					'description' => $description,
					'startDate' => $startDate,
					'endDate' => $endDate
			) );
		} else {
			$id = $this->_getParam ( 'userid', 0 );
			$firstname = $this->_getParam ( 'firstName', "" );
			$lastname = $this->_getParam ( 'lastName', "" );
			$this->view->account = array (
					"id" => $id,
					"firstName" => $firstname,
					"lastName" => $lastname
			);
		}
	}

}

