package org.galaxy.client.api.impl;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipException;

import org.galaxy.client.api.IModelFragmentManagement;
import org.galaxy.client.service.fragmentmanagement.ModelFragmentData;
import org.galaxy.client.service.fragmentmanagement.ModelFragmentManagementProxy;
import org.galaxy.client.service.fragmentmanagement.ressource.ModelFragmentRessource;
import org.galaxy.client.service.fragmentmanagement.ressource.ModelFragmentRessourceUtils;

public class ModelFragmentManagementImpl extends ModelFragmentManagementProxy implements IModelFragmentManagement {

    /**
     * This method handles the packaging of a giving ModelFragment and its resources (xmifile, resourceFiles, propertyFiles).
     * The resulting archive is create inside the fileRoot folder.
     * @param fragment : the ModelFragment to package
     * @param fileRoot : the folder which will contain the created archive and containing all the resources (xmifile, resourceFiles, propertyFiles)
     * @return the resulting archive
     */
    @Override
    public File packageFragment(ModelFragmentRessource fragment, File fileRoot) {

        return ModelFragmentRessourceUtils.packageFragment(fragment, fileRoot);
    }



    /**
     * This method handles the unpackaging of the given archive inside an existing ouputDir 
     * and returns the corresponding Model Fragment
     * @param outputDir : the root folder of the unpackaging result
     * @return the ModelFragment
     * @throws IOException IOException
     * @throws ZipException  ZipException
     */
    @Override
    public File retrieve(String modelFragmentQualifiedName, String version, File outputDir) throws ZipException, IOException {
        ModelFragmentData data = findModelFragmentById(modelFragmentQualifiedName,version);
        String archivePath = outputDir.getAbsolutePath() + File.separator +  data.getName() +"_" + data.getVersion() + ".frg";
        return ModelFragmentRessourceUtils.getFileFromArrayByte(retrieve(modelFragmentQualifiedName,version), archivePath);
    }

    /**
     * retrieves a model fragment identified by a given Id and this dependences and unpackaging it
     * @param fragmentId : the identified ModelFragment
     * @param outputDir : the folder where the ModelFragment is unpackaged
     * @return the resulting ModelFragmentRessource
     * @throws ZipException IOException
     * @throws IOException ZipException
     */
    public List<File> retrieveAll(String modelFragmentQualifiedName, String version, File outputDir) throws ZipException, IOException{
        List<File> result = new ArrayList<File>();

        
        List<ModelFragmentData> dependencies = new ArrayList<ModelFragmentData>();
        List<ModelFragmentData> temp = null;

        List<Integer> resultIds = new ArrayList<Integer>();
        
        ModelFragmentData data = findModelFragmentById(modelFragmentQualifiedName, version);
        result.add(retrieve(modelFragmentQualifiedName, version, outputDir));
        resultIds.add(data.getModelfragmentId());
        
        if(data.getDependencies() != null){
            for (ModelFragmentData depend : data.getDependencies()){
                dependencies.add(depend);           
            }
        }

        while (dependencies.size() > 0){
        	
        	temp = new ArrayList<ModelFragmentData>();
        	
            for (ModelFragmentData depend : dependencies){
                result.add(retrieve(depend.getName(), depend.getVersion(), outputDir));
                resultIds.add(depend.getModelfragmentId());
                
                if(depend.getDependencies() != null){
                    for (ModelFragmentData depend2 : depend.getDependencies()){
                    	if (!resultIds.contains(depend2.getModelfragmentId()))
                    		temp.add(depend2);                    
                    }
                }
            }
            
           dependencies.clear();
           dependencies.addAll(temp);
            
        }


        return result;
    }


    @Override
    public void publish(File fragment) throws IOException {
        publish(ModelFragmentRessourceUtils.getArrayByteFromFile(fragment));;
    }

}
