package org.galaxy.service.projectmanagement;

import java.io.IOException;
import java.util.List;


/**
 * This interface contains services in order to manage projects
 */
public interface IProjectManagement{
	
	/**
	 * Add specified Participant to Project with specified roles
	 * Roles must be roles of project
	 * @param projectId Identifier of project
	 * @param participantId Identifier of participant
	 * @param roleId List of role Identifier
	 * @throws IOException 
	 */
	public void addParticipant(Integer projectId,Integer participantId,List<Integer> roleIds) throws IOException;
	
	
	/**
	 * Remove specified Participant from Project with specified role
	 * Roles must be roles of project
	 * @param projectId Identifier of Project
	 * @param participantId Identifier of Participant
	 * @param roleIds List of role Identifier
	 * @throws IOException 
	 */
	public void removeParticipant(Integer projectId,Integer participantId,List<Integer> roleIds) throws IOException;
	
	
	/**
	 * Add specified Role to specified Project
	 * @param projectId Identifier of project
	 * @param roleId Identifier of role
	 * @throws IOException 
	 */
	public void addRole(Integer projectId,Integer roleId) throws IOException;
	
	
	/**
	 * Add specified Role of specified Project
	 * @param projectId Identifier of project
	 * @param roleId Identifier of role
	 * @throws IOException 
	 */
	public void removeRole(Integer projectId,Integer roleId) throws IOException;

	/**
	 * Find a project identified by  his Identifier
	 * @param projectId Identifier of project
	 * @return Project Data
	 */
	public Project findProjectById(Integer projectId) throws IOException;


	/**
	 * Find a list project identified by  her Identifier
	 * @param projectId Identifier of project
	 * @return Project Data
	 */
	public List<Project> findProjectByIds(List<Integer> projectIds) throws IOException;
	
	/**
	 * List all existing projects.
	 * @return List of projects
	 * @throws IOException 
	 */
	public List<Project> findAllProjects() throws IOException;


	/**
	 * List all project of specified participant.
	 * @return List of project
	 * @throws IOException 
	 */
	public List<Project> findProjectsByParticipant(Integer participantId) throws IOException;
	
	/**
	 *  Add specified participant to Project
	 * @param projectId identifiers of project
	 * @param participantIds List of participant's identifiers
	 * @param roleIds List of role's identifiers
	 * @throws IOException 
	 */
	public void addParticipants(Integer projectId,List<Integer> participantIds,List<Integer> roleIds) throws IOException;
	
	/**
	 * Remove specified participant from Project
	 * @param projectId identifiers of project
	 * @param participantIds
	 * @param roleIds List of role's identifiers
	 * @throws IOException 
	 */
	public void removeParticipants(Integer projectId,List<Integer> participantIds,List<Integer> roleIds) throws IOException;
	
	/**
	 * Add specified roles to Project
	 * @param projectId identifiers of project
	 * @param rolesIds List of role's identifiers
	 * @throws IOException 
	 */
	public void addRoles(Integer projectId,List<Integer> rolesIds) throws IOException;
	
	/**
	 * Remove List of roles to Project
	 * @param projectsId identifiers of project
	 * @param rolesIds identifiers of roles
	 * @throws IOException 
	 */
	public void removeRoles(Integer projectsId,List<Integer> rolesIds) throws IOException;


	/**
	 * Creates a new project in the system .It returns the id of project
	 * @param project Project Data
	 * @return id of project
	 * @throws IOException 
	 */
	public Integer createProject(Project project) throws IOException;


	/**
	 * Delete the given project from the system
	 * @param projectId Identifier of project
	 * @throws IOException 
	 */
	public void  deleteProject(Integer projectId) throws IOException;


	/**
	 * Update the given project from the system
	 * @param project Project Data
	 * @throws IOException 
	 */
	public void updateProject(Project project) throws IOException;


	/**
	 * Delete the given projects from the system
	 * @param projectIds Identifier of projects
	 * @throws IOException 
	 */
	public void deleteProjects(List<Integer> projectIds) throws IOException; 


}
