package org.galaxy.service.workgroupmanagement;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.validation.constraints.NotNull;

import org.galaxy.i18n.Messages;
import org.galaxy.model.EParticipant;
import org.galaxy.model.EWorkgroup;

/**
 * Implementation of IWorkgroupManagement services dedicated to workgroup management 
 */
@Remote
@Stateless
@WebService(targetNamespace = "http://org.galaxy.service.workgoupmanagement/v1/", name = "WorkgroupManagement", portName = "WorkgroupManagementPort", serviceName = "WorkgroupManagementService")
@SOAPBinding(style = SOAPBinding.Style.DOCUMENT, use = SOAPBinding.Use.LITERAL, parameterStyle = SOAPBinding.ParameterStyle.WRAPPED)
public class WorkgroupManagement implements IWorkgroupManagement {

	@PersistenceContext(unitName = "galaxy")
	private EntityManager manager;

	@Override
	@WebMethod
	public void addParticipant(
			@NotNull @WebParam(name = "workgroupId") Integer workgroupId,
			@NotNull @WebParam(name = "participantId") Integer participantId)
			throws IOException {

		if (workgroupId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"workgroupId"));
		} else if (participantId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"participantId"));
		}

		EWorkgroup workgroup = manager.find(EWorkgroup.class, workgroupId);

		if (workgroup == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", workgroupId.toString()));
		}

		EParticipant paticipant = manager.find(EParticipant.class,
				participantId);

		if (paticipant == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Participant", participantId.toString()));
		}

		workgroup.getParticipants().add(paticipant);
		paticipant.getWorkgroups().add(workgroup);
		manager.merge(workgroup);
		manager.merge(paticipant);
	}

	@WebMethod
	@Override
	public void removeParticipant(
			@NotNull @WebParam(name = "workgroupId") Integer workgroupId,
			@NotNull @WebParam(name = "participantId") Integer participantId)
			throws IOException {
		
		
		if (workgroupId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Workgroup"));
		} else if (participantId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Participant"));
		}

		EWorkgroup workgroup = manager.find(EWorkgroup.class, workgroupId);

		if (workgroup == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", workgroupId.toString()));
		}

		EParticipant paticipant = manager.find(EParticipant.class,
				participantId);

		if (paticipant == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Participant", participantId.toString()));
		}

		workgroup.getParticipants().remove(paticipant);
		paticipant.getWorkgroups().remove(workgroup);
		manager.merge(workgroup);
		manager.merge(paticipant);

	}

	@Override
	@WebMethod
	@WebResult(name = "workgroup")
	public Workgroup findWorkgroupById(
			@NotNull @WebParam(name = "workgroupId") Integer workgroupId)
			throws IOException {
		if (workgroupId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Workgroup"));
		} 


		EWorkgroup eworkgroup = manager.find(EWorkgroup.class, workgroupId);

		if (eworkgroup == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Workgroup", workgroupId.toString()));
		}
		
		Workgroup workgroup = new Workgroup();
		workgroup.setName(eworkgroup.getName());
		workgroup.setDescription(eworkgroup.getDescription());
		workgroup.setId(eworkgroup.getWorkgroupId());

		return workgroup;	
	}

	@Override
	@WebMethod
	@WebResult(name = "workgroups")
	public List<Workgroup> findAllWorkgroups() throws IOException {

		List<Workgroup> result = new ArrayList<Workgroup>();
		Query query = manager.createQuery("SELECT w FROM EWorkgroup w");

		for (EWorkgroup eworkgroup : (Collection<EWorkgroup>) query.getResultList()) {
			Workgroup workgroup = new Workgroup();
			workgroup.setName(eworkgroup.getName());
			workgroup.setDescription(eworkgroup.getDescription());
			workgroup.setId(eworkgroup.getWorkgroupId());
			
			result.add(workgroup);
		}

		return result;
	
		
	}

	@Override
	@WebMethod
	@WebResult(name = "workgroups")
	public List<Workgroup> findWorkgroupsByParticipant(
			@NotNull @WebParam(name = "participantId") Integer participantId)
			throws IOException {
		
		List<Workgroup> result = new ArrayList<Workgroup>();
		if (participantId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Participant"));
		}

		EParticipant participant = manager.find(EParticipant.class, participantId);

		if (participant == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", participant.toString()));
		}
		
		for (EWorkgroup eworkgroup : participant.getWorkgroups()) {
			Workgroup workgroup = new Workgroup();
			workgroup.setName(eworkgroup.getName());
			workgroup.setDescription(eworkgroup.getDescription());
			workgroup.setId(eworkgroup.getWorkgroupId());
			
			result.add(workgroup);
		}
		
		return result;
	}

	@Override
	@WebMethod
	public void addParticipants(
			@NotNull @WebParam(name = "workgroupId") Integer workgroupId,
			@NotNull @WebParam(name = "participantIds") List<Integer> participantIds)
			throws IOException {

		if (workgroupId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Workgroup"));
		} 
		
		
		EWorkgroup workgroup = manager.find(EWorkgroup.class, workgroupId);

		if (workgroup == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", workgroupId.toString()));
		}
		
		for(Integer participantId : participantIds){		
			if (participantId == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID",
						"Participant"));
			}
			
			EParticipant paticipant = manager.find(EParticipant.class,
					participantId);

			if (paticipant == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Participant", participantId.toString()));
			}
			workgroup.getParticipants().add(paticipant);
			paticipant.getWorkgroups().add(workgroup);
			manager.merge(paticipant);			
		}

		manager.merge(workgroup);
	}

	@Override
	@WebMethod
	public void removeParticipants(
			@NotNull @WebParam(name = "workgroupId") Integer workgroupId,
			@NotNull @WebParam(name = "participantIds") List<Integer> participantIds)
			throws IOException {
		
		if (workgroupId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Workgroup"));
		} 
		
		EWorkgroup workgroup = manager.find(EWorkgroup.class, workgroupId);

		if (workgroup == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", workgroupId.toString()));
		}
		
		for(Integer participantId : participantIds){
			if (participantId == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID",
						"Participant"));
			}

		

			EParticipant paticipant = manager.find(EParticipant.class,
					participantId);

			if (paticipant == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Participant", participantId.toString()));
			}

			workgroup.getParticipants().remove(paticipant);
			paticipant.getWorkgroups().remove(workgroup);
			manager.merge(paticipant);
		}
		
		manager.merge(workgroup);

	}

	@Override
	@WebMethod
	@WebResult(name = "workgroups")
	public List<Workgroup> findWorkgroupsByIds(
			@NotNull @WebParam(name = "workgroupsIds") List<Integer> workgroupsIds)
			throws IOException {
		
		List<Workgroup> result  = new ArrayList<Workgroup>();
		
		for(Integer workgroupId : workgroupsIds){
			if (workgroupId == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID",
						"Workgroup"));
			} 


			EWorkgroup eworkgroup = manager.find(EWorkgroup.class, workgroupId);

			if (eworkgroup == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Workgroup", workgroupId.toString()));
			}
			
			Workgroup workgroup = new Workgroup();
			workgroup.setName(eworkgroup.getName());
			workgroup.setDescription(eworkgroup.getDescription());
			workgroup.setId(eworkgroup.getWorkgroupId());
			
			result.add(workgroup);
		}
		
		return result;
	}

	@Override
	@WebMethod
	@WebResult(name = "workgroupId")
	public Integer createWorkgroup(
			@NotNull @WebParam(name = "workgroup") Workgroup workgroup)
			throws IOException {

		// Parameter Validation
		if (workgroup == null) {
			throw new IOException(Messages.getString("GALAXY_PV_NULL",
					"Workgroup"));
		}

		EWorkgroup eworkgroup = new EWorkgroup();
		eworkgroup.setName(workgroup.getName());
		eworkgroup.setDescription(workgroup.getDescription());
		manager.persist(eworkgroup);

		return eworkgroup.getWorkgroupId();
	}

	@Override
	@WebMethod
	public void deleteWorkgroup(
			@NotNull @WebParam(name = "workgroupId") Integer workgroupId)
			throws IOException {

		if (workgroupId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Workgroup"));
		}

		EWorkgroup workgroup = manager.find(EWorkgroup.class, workgroupId);

		if (workgroup == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Workgroup", workgroupId.toString()));
		}

		manager.remove(workgroup);

	}

	@Override
	@WebMethod
	@WebResult(name = "workgroupIds")
	public void deleteWorkgroups(
			@NotNull @WebParam(name = "workgroupIds") List<Integer> workgroupIds)
			throws IOException {
		for (Integer workgroupId : workgroupIds) {
			if (workgroupId == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID",
						"Workgroup"));
			}

			EWorkgroup workgroup = manager.find(EWorkgroup.class, workgroupId);

			if (workgroup == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Workgroup", workgroupId.toString()));
			}

			manager.remove(workgroup);
		}
	}

	@Override
	@WebMethod
	public void updateWorkgroup(
			@NotNull @WebParam(name = "workgroup") Workgroup workgroup)
			throws IOException {

		// Parameter Validation
		if (workgroup == null) {
			throw new IOException(Messages.getString("GALAXY_PV_NULL",
					"Workgroup"));
		} else if (workgroup.getId() == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
		}

		
		if (findWorkgroupById(workgroup.getId()) == null ) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Workgroup", workgroup.getId().toString()));
		}
		
		EWorkgroup eworkgroup = new EWorkgroup();
		eworkgroup.setName(workgroup.getName());
		eworkgroup.setDescription(workgroup.getDescription());
		eworkgroup.setWorkgroupId(workgroup.getId());

	

		manager.merge(eworkgroup);

	}

}
