package org.galaxy.utility.filerepository;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.ejb.Remote;
import javax.ejb.Stateless;

import org.galaxy.i18n.Messages;


/**
 * Implementation of IFileRepository services dedicated to management of physical ModelFragment repository  management 
 */
@Remote
@Stateless
public class FileRepository implements IFileRepository{

	@Override
	public void deleteFile(String fragmentname, String fragmentversion)
			throws IOFragmentException {
		String localpath = calculateLocalPath(fragmentname,fragmentversion);
		File fragment = new File(localpath);
		
		if(!fragment.exists()){
			throw new  IOFragmentException(Messages.getString("MISSING_FRAGMENT", fragmentname,fragmentversion));
		}
		
		fragment.delete();
		
		File directory = fragment.getParentFile();
		
		if(directory.list().length == 0){
			directory.delete();
		}
	}

	@Override
	public String calculateLocalPath(String fragmentname, String fragmentversion) {
		String repositorypath = System.getProperty("org.galaxy.repositorypath");
		return repositorypath + "/" +fragmentname+ "/" +fragmentname +"_" + fragmentversion +".frg";
	}

	@Override
	public void storeFile(File temporaryfile,String fragmentname, String fragmentversion) throws IOFragmentException {
		
		String localpath = calculateLocalPath(fragmentname,fragmentversion);
		File fragment = new File(localpath);
		
		if(fragment.exists()){
			throw new  IOFragmentException(Messages.getString("EXISTING_FRAGMENT",  fragmentname,fragmentversion));
		}
		
		File repository = fragment.getParentFile();
		repository.mkdirs();
		
		copy(temporaryfile,fragment);
		
	}

	@Override
	public String retiveFile(String fragmentname, String fragmentversion)
			throws IOFragmentException {
		
		String localpath = calculateLocalPath(fragmentname,fragmentversion);
		File fragment = new File(localpath);
		
		if(!fragment.exists()){
			throw new  IOFragmentException(Messages.getString("MISSING_FRAGMENT", fragmentname,fragmentversion));
		}
		
		return fragment.getAbsolutePath();
	}
	
	private void copy(File source,File target) throws IOFragmentException{
		if (!source.equals(target) && source.canRead() && (!target.exists() || target.canWrite())) {
			try {
				if (source.isFile()) {
					target.getParentFile().mkdirs();
					java.io.FileInputStream sourceFile = new java.io.FileInputStream(source);
					try {
						java.io.FileOutputStream destinationFile = null;
						try {
							destinationFile = new FileOutputStream(target);

							byte buffer[] = new byte[512 * 1024];
							int nbLecture;

							while ((nbLecture = sourceFile.read(buffer)) != -1) {
								destinationFile.write(buffer, 0, nbLecture);
							}
						} finally {
						    if (destinationFile != null) {
						        destinationFile.close();
							}
						}
					} finally {
						sourceFile.close();
					}
				} 
			} catch (IOException e) {
				throw new IOFragmentException(Messages.getString("FILE_COPPY_ERROR",target.getName()));
			}
		}
	}

}
